/*
 * Copyright (C) 2023-2025 Xeme Developers
 *
 * Xeme is the legal property of its developers, whose names are too
 * numerous to list here. Please refer to the AUTHORS file distributed
 * with this source distribution
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef XEME_CONNECTION_H
#define XEME_CONNECTION_H

#include <glib.h>
#include <glib-object.h>

#include <gio/gio.h>

#include "xemeinputstream.h"
#include "xemeoutputstream.h"
#include "xemeversion.h"

G_BEGIN_DECLS

/**
 * XemeConnection:
 *
 * A client to server connection.
 *
 * This object manages the connection to the server, handles feature
 * negotiation including STARTTLS, stream restarts, and other things that are
 * part of XMPP Core.
 *
 * Since: 0.1
 */

#define XEME_TYPE_CONNECTION (xeme_connection_get_type())

XEME_AVAILABLE_IN_0_1
G_DECLARE_FINAL_TYPE(XemeConnection, xeme_connection, XEME, CONNECTION,
                     GObject)

/**
 * XemeTlsMode:
 * @XEME_TLS_MODE_DIRECT_TLS: Use direct TLS.
 * @XEME_TLS_MODE_START_TLS: Use start TLS.
 * @XEME_TLS_MODE_NONE: Don't use TLS. This should really only be used in rare
 *                      circumstances like development.
 *
 * An enumeration representing which mode of TLS to use.
 *
 * Since: 0.1
 */
typedef enum {
	XEME_TLS_MODE_DIRECT_TLS,
	XEME_TLS_MODE_START_TLS,
	XEME_TLS_MODE_NONE,
} XemeTlsMode;

/**
 * xeme_connection_new:
 * @jid: (not nullable): The JID of the originator of this connection.
 *
 * Creates a new connection.
 *
 * If @jid includes a
 * [resource](https://www.rfc-editor.org/rfc/rfc6122#section-2), it will be
 * parsed out and stored in [property@Connection:resource].
 *
 * Returns: (transfer full): The new instance.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeConnection *xeme_connection_new(const char *jid);

/**
 * xeme_connection_get_jid:
 * @connection: The instance.
 *
 * Gets the bare [JID](https://www.rfc-editor.org/rfc/rfc6122#section-2) from
 * @connection.
 *
 * Returns: The bare jid.
 *
 * Since: 0.1
 */
const char *xeme_connection_get_jid(XemeConnection *connection);

/**
 * xeme_connection_get_resource:
 * @connection: The instance.
 *
 * Gets the resource from @connection.
 *
 * This may be %NULL in rare circumstances. However, @connection will request a
 * JID from the server during connection if this is %NULL.
 *
 * Returns: (transfer none) (nullable): The resource if set otherwise %NULL.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_connection_get_resource(XemeConnection *connection);

/**
 * xeme_connection_set_resource:
 * @connection: The instance.
 * @resource: (not nullable): The new resource.
 *
 * Sets the resource of @connection to @resource.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_connection_set_resource(XemeConnection *connection, const char *resource);

/**
 * xeme_connection_get_cancellable:
 * @connection: The instance.
 *
 * Gets the [class@Gio.Cancellable] for @connection.
 *
 * Returns: (transfer none) (nullable): The canncellable if one is set,
 *          otherwise %NULL.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
GCancellable *xeme_connection_get_cancellable(XemeConnection *connection);

/**
 * xeme_connection_get_server:
 * @connection: The instance.
 *
 * Gets the explicit server that @connection should connect to.
 *
 * See [method@Connection.connect_async] for more information.
 *
 * Returns: (nullable): The explicit server to connect to or %NULL.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_connection_get_server(XemeConnection *connection);

/**
 * xeme_connection_set_server:
 * @connection: The instance.
 * @server: (nullable): The new explicit server to connect to.
 *
 * Sets the explicit server to connect to.
 *
 * Passing %NULL for @server will unset any previous value.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_connection_set_server(XemeConnection *connection, const char *server);

/**
 * xeme_connection_get_port:
 * @connection: The instance.
 *
 * Gets the port from @connection.
 *
 * Returns: The port to use.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
guint16 xeme_connection_get_port(XemeConnection *connection);

/**
 * xeme_connection_set_port:
 * @connection: The instance.
 * @port: The new port value.
 *
 * Sets the port to use for @connection to @port.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_connection_set_port(XemeConnection *connection, guint16 port);

/**
 * xeme_connection_get_tls_mode:
 * @connection: The instance.
 *
 * Gets the [enum@TlsMode] from @connection.
 *
 * Returns: The TLS mode to use.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeTlsMode xeme_connection_get_tls_mode(XemeConnection *connection);

/**
 * xeme_connection_set_tls_mode:
 * @connection: The instance.
 * @tls_mode: The new TLS mode.
 *
 * Sets the [enum@TlsMode] of @connection to @tls_mode.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_connection_set_tls_mode(XemeConnection *connection, XemeTlsMode tls_mode);

/**
 * xeme_connection_connect_async:
 * @connection: The instance.
 * @input: The [class@InputStream] to use.
 * @resolver: (transfer full) (nullable): An optional proxy resolver.
 * @cancellable: (transfer none) (nullable): An optional cancellable.
 * @callback: (scope async) (nullable): A callback function.
 * @data: (nullable): User data pass to @callback.
 *
 * Starts the connection process.
 *
 * If [property@Connection:server] is not %NULL, then this will try to connect
 * directly to that server on the port specified by [property@Connection:port].
 *
 * If [property@Connection:server] is %NULL then DNS SRV records will be used
 * to connect per
 * [RFC 6120](https://www.rfc-editor.org/rfc/rfc6120#section-3.2.4).
 *
 * If [property@Connection:tls-mode] is set to `direct-tls`, then the
 * `_xmpps-client` service will be queried. If the DNS SRV lookup does not
 * return a record, an attempt will be made to connect directly to the domain
 * part of [property@Connection:jid] on the port specified by
 * [property@Connection:port].
 *
 * If [property@Connection:tls-mode] is set to `starttls` or `none` then the
 * `_xmpp-client` service will be queried. If the DNS SRV lookup does not
 * return a record, an attempt will be made to connect directly to the domain
 * part of [property@Connection:jid] on the port specified by
 * [property@Connection:port].
 *
 * If [property@Connection:tls-mode] is set to `starttls` then the connection
 * will attempt to upgrade to TLS during the connection process. If this fails,
 * the connection will be terminated.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_connection_connect_async(XemeConnection *connection, XemeInputStream *input, GProxyResolver *resolver, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer data);

/**
 * xeme_connection_connect_finish:
 * @connection: The instance.
 * @result: The result passed to the callback.
 * @error: A return address for a #GError.
 *
 * Finishes a previous call to [method@Connection.connect_async].
 *
 * This should be call from the callback function of
 * [method@Connection.connect_async] to get the result of the connection
 * attempt.
 *
 * Returns: (transfer full): A new [class@OutputStream] or %NULL if the
 *          connection failed.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeOutputStream *xeme_connection_connect_finish(XemeConnection *connection, GAsyncResult *result, GError **error);

/**
 * xeme_connection_close:
 * @connection: The instance.
 * @error: (nullable): A return address for a #GError.
 *
 * Attempts to close the connection.
 *
 * Returns: %TRUE if sucessfull, otherwise %FALSE with @error possibly set.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
gboolean xeme_connection_close(XemeConnection *connection, GError **error);

G_END_DECLS

#endif /* XEME_CONNECTION_H */
