/*
 * Copyright (C) 2023-2025 Xeme Developers
 *
 * Xeme is the legal property of its developers, whose names are too
 * numerous to list here. Please refer to the AUTHORS file distributed
 * with this source distribution
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef XEME_EXTENSION_H
#define XEME_EXTENSION_H

#include <glib.h>
#include <glib-object.h>

#include <gio/gio.h>

#include "xemeversion.h"

G_BEGIN_DECLS

/**
 * XemeExtension:
 *
 * The base class for
 * [extension elements](https://datatracker.ietf.org/doc/html/rfc6120#section-8.4).
 *
 * Since: 0.1
 */

#define XEME_TYPE_EXTENSION (xeme_extension_get_type())

XEME_AVAILABLE_IN_0_1
G_DECLARE_DERIVABLE_TYPE(XemeExtension, xeme_extension, XEME, EXTENSION,
                         GObject)

#include "xemeinputstream.h"

struct _XemeExtensionClass {
	/*< private >*/
	GObjectClass parent;

	/*< public >*/
	void (*parse_start)(XemeExtension *extension, XemeInputStream *input, GMarkupParseContext *context);
	void (*parse_end)(XemeExtension *extension, GMarkupParseContext *context);
	char *(*serialize)(XemeExtension *extension);

	/*< private >*/
	gpointer reserved[8];
};

/**
 * xeme_extension_get_element:
 * @extension: The instance.
 *
 * Gets the name of the element from @extension.
 *
 * Returns: The name of the element of the extension.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_extension_get_element(XemeExtension *extension);

/**
 * xeme_extension_get_namespace:
 * @extension: The instance.
 *
 * Gets the namespace from @extension.
 *
 * Returns: The namespace of the extension.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_extension_get_namespace(XemeExtension *extension);

/**
 * xeme_extension_parse_start:
 * @extension: The instance.
 * @input: The [class@XemeInputStream] that is being parsed.
 * @context: The [struct@GLib.MarkupParseContext] of the parser.
 *
 * Starts parsing an extension.
 *
 * This calls [vfunc@Extension.parse_start] which should use @context in a call
 * to [method@GLib.MarkupParseContext.push] with its own parser to parse the
 * data and set @extension appropriately.
 *
 * Note: This method should be called from a start_element implementation of a
 * [struct@GLib.MarkupParser].
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_extension_parse_start(XemeExtension *extension, XemeInputStream *input, GMarkupParseContext *context);

/**
 * xeme_extension_parse_end:
 * @extension: The instance.
 * @context: The [struct@GLib.MarkupParseContext] of the parser.
 *
 * Finishes a previous call to [method@XemeExtension.parse_start].
 *
 * Note: This method should be called from an end_element implementation of a
 * [struct@GLib.MarkupParser].
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_extension_parse_end(XemeExtension *extension, GMarkupParseContext *context);

/**
 * xeme_extension_serialize:
 * @extension: The instance.
 *
 * Serializes @extension into XML.
 *
 * This calls [vfunc@Extension.serialize] to do the actual serialization.
 *
 * Returns: (transfer full): The XML data as a string for @extension.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
char *xeme_extension_serialize(XemeExtension *extension);

G_END_DECLS

#endif /* XEME_EXTENSION_H */

