/*
 * Copyright (C) 2023-2025 Xeme Developers
 *
 * Xeme is the legal property of its developers, whose names are too
 * numerous to list here. Please refer to the AUTHORS file distributed
 * with this source distribution
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef XEME_EXTENSION_MAP_H
#define XEME_EXTENSION_MAP_H

#include <glib.h>
#include <glib-object.h>

#include <gio/gio.h>

#include "xemeextension.h"
#include "xemeversion.h"

G_BEGIN_DECLS

/**
 * XemeExtensionMap:
 *
 * A collection for keeping track of and handling
 * [extension elements](https://datatracker.ietf.org/doc/html/rfc6120#section-8.4).
 *
 * Since: 0.1
 */

#define XEME_TYPE_EXTENSION_MAP (xeme_extension_map_get_type())

XEME_AVAILABLE_IN_0_1
G_DECLARE_FINAL_TYPE(XemeExtensionMap, xeme_extension_map, XEME, EXTENSION_MAP,
                     GObject)

/**
 * xeme_extension_map_new:
 *
 * Creates a new extension map.
 *
 * Returns: (transfer full): The new instance.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeExtensionMap *xeme_extension_map_new(void);

/**
 * xeme_extension_map_add:
 * @map: The instance.
 * @extension: The extension to add.
 *
 * Adds @extension to @map.
 *
 * Returns: %TRUE if @extension was not already in @map, otherwise %FALSE.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
gboolean xeme_extension_map_add(XemeExtensionMap *map, XemeExtension *extension);

/**
 * xeme_extension_map_remove:
 * @map: The instance.
 * @extension: The extension to remove.
 *
 * Removes @extension from @map.
 *
 * Returns: %TRUE if @extension was removed from @map, otherwise %FALSE.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
gboolean xeme_extension_map_remove(XemeExtensionMap *map, XemeExtension *extension);

/**
 * xeme_extension_map_lookup:
 * @map: The instance.
 * @element: (not nullable): The name of the element of the extension.
 * @namespace: The namespace of the extension.
 *
 * Look if an extension matching @element and @namespace has been added to
 * @map.
 *
 * Returns: (transfer none): The extension if found, otherwise %NULL.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeExtension *xeme_extension_map_lookup(XemeExtensionMap *map, const char *element, const char *namespace);

/**
 * xeme_extension_map_get_size:
 * @map: The instance.
 *
 * Gets the number of extensions that @map knows about.
 *
 * Returns: The number of extensions added to @map.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
guint xeme_extension_map_get_size(XemeExtensionMap *map);

/**
 * xeme_extension_map_parse_start:
 * @map: The instance.
 * @input: The [class@InputStream] that is being parsed.
 * @context: The [struct@GLib.MarkupParseContext] of the parser.
 *
 * Walks through all of the elements in @context. If extension elements are
 * found in @map, then their parse methods will be called and processed.
 *
 * This should only be called from a start_element implementation of a
 * [struct@GLib.MarkupParser].
 *
 * Note: This only processes extension elements, if you have mixed context, you
 * will need to use [method@ExtensionMap.lookup] to determine if an element is
 * an extension element and then call the [method@Extension.parse_start] in
 * the start_element implementation as well as the [method@Extension.parse_end]
 * in the end_element implementation.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_extension_map_parse_start(XemeExtensionMap *map, XemeInputStream *input, GMarkupParseContext *context);

/**
 * xeme_extension_map_parse_end:
 * @map: The instance.
 * @context: The [struct@GLib.MarkupParseContext] for the parser.
 *
 * Finishes a previous call to [method@ExtensionMap.parse_start].
 *
 * This method should be called from an end_element implementation of a
 * [struct@GLib.MarkupParser].
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_extension_map_parse_end(XemeExtensionMap *map, GMarkupParseContext *context);

G_END_DECLS

#endif /* XEME_EXTENSION_MAP_H */
