/*
 * Copyright (C) 2023-2025 Xeme Developers
 *
 * Xeme is the legal property of its developers, whose names are too
 * numerous to list here. Please refer to the AUTHORS file distributed
 * with this source distribution
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef XEME_LOCALIZED_STRING_H
#define XEME_LOCALIZED_STRING_H

#include <glib.h>
#include <glib-object.h>

#include "xemeversion.h"

G_BEGIN_DECLS

/**
 * XemeLocalizedString:
 *
 * A string that has a language associated with it.
 *
 * Since: 0.1
 */

#define XEME_TYPE_LOCALIZED_STRING (xeme_localized_string_get_type())

XEME_AVAILABLE_IN_0_1
G_DECLARE_FINAL_TYPE(XemeLocalizedString, xeme_localized_string, XEME,
                     LOCALIZED_STRING, GObject)

/**
 * xeme_localized_string_equal:
 * @l10nstr2: (nullable): the second instance
 *
 * Checks if two localized strings are equal.
 *
 * This does not take case folding into consideration.
 *
 * Returns: true if the two languages and strings are equal.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
gboolean xeme_localized_string_equal(XemeLocalizedString *l10nstr1, XemeLocalizedString *l10nstr2);

/**
 * xeme_localized_string_get_language:
 *
 * Gets the language of the string.
 *
 * Returns: (nullable): The language.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_localized_string_get_language(XemeLocalizedString *str);

/**
 * xeme_localized_string_get_string:
 *
 * Gets the string.
 *
 * Returns: (nullable): The string.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_localized_string_get_string(XemeLocalizedString *str);

/**
 * xeme_localized_string_new:
 * @lang: (nullable): the language for the string
 * @str: the string.
 *
 * Creates a new localized string.
 *
 * Returns: (transfer full): The string.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeLocalizedString *xeme_localized_string_new(const char *lang, const char *str);

/**
 * xeme_localized_string_set_language:
 * @lang: (nullable): the language
 *
 * Sets the language for a string.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_localized_string_set_language(XemeLocalizedString *l10nstr, const char *lang);

/**
 * xeme_localized_string_set_string:
 * @str: the string
 *
 * Sets the string.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_localized_string_set_string(XemeLocalizedString *l10nstr, const char *str);

G_END_DECLS

#endif /* XEME_LOCALIZED_STRING_H */
