/*
 * Copyright (C) 2023-2025 Xeme Developers
 *
 * Xeme is the legal property of its developers, whose names are too
 * numerous to list here. Please refer to the AUTHORS file distributed
 * with this source distribution
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef XEME_MESSAGE_H
#define XEME_MESSAGE_H

#include <glib.h>
#include <glib-object.h>

#include <gio/gio.h>

#include "xemelocalizedstring.h"
#include "xemestanza.h"
#include "xemeversion.h"

G_BEGIN_DECLS

/**
 * XemeMessageType:
 * @XEME_MESSAGE_TYPE_UNKNOWN: the message type is unknown
 * @XEME_MESSAGE_TYPE_CHAT: the message is for a chat
 * @XEME_MESSAGE_TYPE_ERROR: the message is for an error
 * @XEME_MESSAGE_TYPE_GROUPCHAT: the message is for a groupchat
 * @XEME_MESSAGE_TYPE_HEADLINE: the message is a headline
 * @XEME_MESSAGE_TYPE_NORMAL: a normal message
 *
 * An enumeration to describe the type of a message.
 *
 * See https://datatracker.ietf.org/doc/html/rfc6121#section-5.2.2 for more
 * information.
 *
 * Since: 0.1
 */
typedef enum {
	XEME_MESSAGE_TYPE_UNKNOWN XEME_AVAILABLE_ENUMERATOR_IN_0_1 = 0,
	XEME_MESSAGE_TYPE_CHAT XEME_AVAILABLE_ENUMERATOR_IN_0_1,
	XEME_MESSAGE_TYPE_ERROR XEME_AVAILABLE_ENUMERATOR_IN_0_1,
	XEME_MESSAGE_TYPE_GROUPCHAT XEME_AVAILABLE_ENUMERATOR_IN_0_1,
	XEME_MESSAGE_TYPE_HEADLINE XEME_AVAILABLE_ENUMERATOR_IN_0_1,
	XEME_MESSAGE_TYPE_NORMAL XEME_AVAILABLE_ENUMERATOR_IN_0_1,
} XemeMessageType;

/**
 * XemeMessage:
 *
 * An XMPP Message stanza.
 *
 * Since: 0.1
 */

#define XEME_TYPE_MESSAGE (xeme_message_get_type())

XEME_AVAILABLE_IN_0_1
G_DECLARE_FINAL_TYPE(XemeMessage, xeme_message, XEME, MESSAGE, XemeStanza)

/**
 * xeme_message_add_body:
 * @language: (nullable): the language
 * @content: the content
 *
 * Add a body to the message.
 *
 * If a body already exists with @language, it will be replaced with the new
 * @content.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_message_add_body(XemeMessage *message, const char *language, const char *content);

/**
 * xeme_message_add_subject:
 * @language: (nullable): the language
 * @content: the content
 *
 * Add a subject to the message.
 *
 * If a subject already exists with @language, it will be replaced with the new
 * @content.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_message_add_subject(XemeMessage *message, const char *language, const char *content);

/**
 * xeme_message_get_bodies:
 *
 * Gets all of the bodies for the message.
 *
 * Returns: (transfer none): the bodies.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
GListModel *xeme_message_get_bodies(XemeMessage *message);

/**
 * xeme_message_get_body:
 * @language: (nullable): the language
 *
 * Gets the body matching the given language.
 *
 * Returns: (transfer full) (nullable): the body, if found.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeLocalizedString *xeme_message_get_body(XemeMessage *message, const char *language);

/**
 * xeme_message_get_subject:
 * @language: (nullable): the language
 *
 * Gets the subject matching the given language.
 *
 * Returns: (transfer full) (nullable): the subject, if found.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeLocalizedString *xeme_message_get_subject(XemeMessage *message, const char *language);

/**
 * xeme_message_get_subjects:
 *
 * Gets all of the subjects for the message.
 *
 * Returns: (transfer none): the subjects.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
GListModel *xeme_message_get_subjects(XemeMessage *message);

/**
 * xeme_message_get_message_type:
 *
 * Gets the type of the message.
 *
 * Returns: The type of the message.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeMessageType xeme_message_get_message_type(XemeMessage *message);

/**
 * xeme_message_set_message_type:
 * @type: the new type
 *
 * Sets the type of a message.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_message_set_message_type(XemeMessage *message, XemeMessageType type);

/**
 * xeme_message_get_thread:
 * @message: The instance.
 *
 * Gets the thread identifier of @message.
 *
 * Returns: (transfer none) (nullable): The thread id.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_message_get_thread(XemeMessage *message);

/**
 * xeme_message_get_thread_parent:
 * @message: The instance.
 *
 * Gets the thread parent identifier of @message.
 *
 * Returns: (transfer none) (nullable): The thread id.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_message_get_thread_parent(XemeMessage *message);

/**
 * xeme_message_new:
 *
 * Creates a new [class@Message] instance.
 *
 * returns: (transfer full) (type XemeMessage): The new message.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeStanza *xeme_message_new(void);

/**
 * xeme_message_remove_body:
 * @language: the language
 *
 * Removes the body with given language.
 *
 * Returns: true if a body was found and removed.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
gboolean xeme_message_remove_body(XemeMessage *message, const char *language);

/**
 * xeme_message_remove_subject:
 * @language: the language
 *
 * Removes the subject with given language.
 *
 * Returns: true if a subject was found and removed.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
gboolean xeme_message_remove_subject(XemeMessage *message, const char *language);

/**
 * xeme_message_set_thread:
 * @message: The instance.
 * @thread: (nullable): The new thread identifier.
 *
 * Sets the thread identifier of @message.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_message_set_thread(XemeMessage *message, const char *thread);

/**
 * xeme_message_set_thread_parent:
 * @message: The instance.
 * @thread_parent: (nullable): The new thread parent identifier.
 *
 * Sets the thread parent identifier of @message.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_message_set_thread_parent(XemeMessage *message, const char *thread_parent);

G_END_DECLS

#endif /* XEME_MESSAGE_H */
