/*
 * Copyright (C) 2023-2025 Xeme Developers
 *
 * Xeme is the legal property of its developers, whose names are too
 * numerous to list here. Please refer to the AUTHORS file distributed
 * with this source distribution
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef XEME_OUTPUT_STREAM_H
#define XEME_OUTPUT_STREAM_H

#include <glib.h>
#include <glib-object.h>

#include <gio/gio.h>

#include "xemestream.h"
#include "xemeversion.h"

G_BEGIN_DECLS

/**
 * XemeOutputStream:
 *
 * An XMPP output stream. This is used to send stanzas to the other end of the
 * connection.
 *
 * Since: 0.1
 */

#define XEME_TYPE_OUTPUT_STREAM (xeme_output_stream_get_type())

XEME_AVAILABLE_IN_0_1
G_DECLARE_FINAL_TYPE(XemeOutputStream, xeme_output_stream, XEME, OUTPUT_STREAM, XemeStream)

/**
 * xeme_output_stream_new:
 * @cancellable: (nullable): A [class@Gio.Cancellable] for the stream.
 *
 * Creates a new output stream.
 *
 * Returns: (transfer full): The new instance.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
XemeOutputStream *xeme_output_stream_new(GCancellable *cancellable);

/**
 * xeme_output_stream_start:
 * @stream: The instance.
 * @output: (transfer none): A [class@Gio.OutputStream] for writing data to the
 *          server.
 * @error: (nullable): A return address for a #GError.
 *
 * Initiates @output as an outgoing XMPP stream. It is the responsibility of
 * the caller to establish any necessary connections as this writes raw XMPP
 * data to @output.
 *
 * Output is handled asynchronously so this function will always return
 * immediately. If setup and processing was successful, %TRUE will be returned,
 * otherwise %FALSE will be returned with @error set.
 *
 * Returns: %TRUE on success, otherwise %FALSE with @error set.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
gboolean xeme_output_stream_start(XemeOutputStream *stream, GOutputStream *output, GError **error);

/**
 * xeme_output_stream_restart:
 * @stream: The instance.
 * @error: (nullable): A return address for a #GError.
 *
 * Restarts the stream per
 * [RFC 6120 4.3.3](https://www.rfc-editor.org/rfc/rfc6120#section-4.3.3).
 *
 * This uses the existing connections but will resend the XML Declaration and
 * the initial `stream` element. This is used during the initial negotiation
 * with the server and probably shouldn't be called otherwise.
 *
 * Returns: %TRUE on success, otherwise %FALSE with @error set.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
gboolean xeme_output_stream_restart(XemeOutputStream *stream, GError **error);

/**
 * xeme_output_stream_stop:
 * @stream: The instance.
 * @error: (nullable): A return address for a #GError.
 *
 * Stops the stream by sending the ending `</stream>` if possible and then
 * closes the output stream.
 *
 * Returns: %TRUE on success, otherwise %FALSE with @error possibly set.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
gboolean xeme_output_stream_stop(XemeOutputStream *stream, GError **error);

G_END_DECLS

#endif /* XEME_OUTPUT_STREAM_H */
