/*
 * Copyright (C) 2023-2025 Xeme Developers
 *
 * Xeme is the legal property of its developers, whose names are too
 * numerous to list here. Please refer to the AUTHORS file distributed
 * with this source distribution
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef XEME_STANZA_H
#define XEME_STANZA_H

#include <glib.h>
#include <glib-object.h>

#include "xemeversion.h"

G_BEGIN_DECLS

#define XEME_TYPE_STANZA (xeme_stanza_get_type())

/**
 * XemeStanza:
 *
 * The base class for XMPP stanzas including Message, IQ, and Presence.
 *
 * Since: 0.1
 */

XEME_AVAILABLE_IN_0_1
G_DECLARE_DERIVABLE_TYPE(XemeStanza, xeme_stanza, XEME, STANZA, GObject)

struct _XemeStanzaClass {
	/*< private >*/
	GObjectClass parent;

	/*< public >*/
	char *(*marshal)(XemeStanza *stanza, GError **error);

	/*< private >*/
	gpointer reserved[8];
};

/**
 * xeme_stanza_get_content_type:
 * @stanza: The instance.
 *
 * Gets the content type of @stanza.
 *
 * Returns: (nullable): The content type of @stanza.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_stanza_get_content_type(XemeStanza *stanza);

/**
 * xeme_stanza_set_content_type:
 * @stanza: The instance.
 * @content_type: (not nullable): The new content type for the stanza.
 *
 * Sets the content type of @stanza.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_stanza_set_content_type(XemeStanza *stanza, const char *content_type);

/**
 * xeme_stanza_get_from:
 * @stanza: The instance.
 *
 * Gets the sender of @stanza.
 *
 * Returns: (nullable): The sender of @stanza.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_stanza_get_from(XemeStanza *stanza);

/**
 * xeme_stanza_set_from:
 * @stanza: The instance.
 * @from: (not nullable): Who sent this stanza.
 *
 * Sets the sender of @stanza.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_stanza_set_from(XemeStanza *stanza, const char *from);

/**
 * xeme_stanza_get_id:
 * @stanza: The instance.
 *
 * Gets the id of @stanza.
 *
 * Returns: (nullable): The identifier of the stanza.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_stanza_get_id(XemeStanza *stanza);

/**
 * xeme_stanza_set_id:
 * @stanza: The instance.
 * @id: (not nullable): The new identifier.
 *
 * Sets the identifier of @stanza.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_stanza_set_id(XemeStanza *stanza, const char *id);

/**
 * xeme_stanza_get_to:
 * @stanza: The instance.
 *
 * Gets the recipient of @stanza.
 *
 * Returns: (nullable): The recipient of this stanza.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_stanza_get_to(XemeStanza *stanza);

/**
 * xeme_stanza_set_to:
 * @stanza: The instance.
 * @to: (not nullable): Who this stanza is addressed to.
 *
 * Sets the recipient of @stanza.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_stanza_set_to(XemeStanza *stanza, const char *to);

/**
 * xeme_stanza_get_xml_lang:
 * @stanza: The instance.
 *
 * Gets the XML Lang of @stanza.
 *
 * Returns: (nullable): The XML lang of @stanza.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
const char *xeme_stanza_get_xml_lang(XemeStanza *stanza);

/**
 * xeme_stanza_set_xml_lang:
 * @stanza: The instance.
 * @xml_lang: (nullable): The new XML Lang.
 *
 * Sets the XML Lang of @stanza.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
void xeme_stanza_set_xml_lang(XemeStanza *stanza, const char *xml_lang);

/**
 * xeme_stanza_marshal:
 * @stanza: The instance.
 * @error: (nullable): A return address for a #GError.
 *
 * Marshals @stanza into a string. If there is an error @error will be set and
 * %NULL will be returned.
 *
 * Returns: (transfer full) (nullable): The marshaled stanza as a string or
 *          %NULL with error set on failure.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
char *xeme_stanza_marshal(XemeStanza *stanza, GError **error);

/**
 * xeme_stanza_marshal_attributes:
 * @stanza: The instance.
 *
 * Marshals the common attributes: id, to, from, type, and xml:lang to a
 * string.
 *
 * If none of the attributes are set, %NULL will be returned. If at least one
 * attribute is set, the returned string will not have any padding so you will
 * have to add your own.
 *
 * Returns: (transfer full) (nullable): The attributes marshaled as a string.
 *
 * Since: 0.1
 */
XEME_AVAILABLE_IN_0_1
char *xeme_stanza_marshal_attributes(XemeStanza *stanza);

G_END_DECLS

#endif /* XEME_STANZA_H */
